@@ -1,7 +1,7 @@
 /* Trampoline test */
 
 /*
- * Copyright 1995-1999, 2001-2002, 2004-2005 Bruno Haible, <bruno@clisp.org>
+ * Copyright 1995-1999, 2001-2002, 2004-2006 Bruno Haible, <bruno@clisp.org>
  *
  * This is free software distributed under the GNU General Public Licence
  * described in the file COPYING. Contact the author if you don't have this
@@ -35,8 +35,12 @@ int f (int x)
 {
 #ifdef __GNUC__
 #ifdef __m68k__
+#ifdef __NetBSD__
+register void* env __asm__("a1");
+#else
 register void* env __asm__("a0");
 #endif
+#endif
 #ifdef __mips__
 register void* env __asm__("$2");
 #endif
@@ -58,7 +62,7 @@ register void* env __asm__("%r29");
 #ifdef __arm__
 register void* env __asm__("r12");
 #endif
-#ifdef __rs6000__
+#ifdef __powerpc__
 #ifdef __NetBSD__
 register void* env __asm__("r13");
 #else
