@@ -128,11 +128,12 @@
           (dotimes (y 20)
             (unless (and (= (field x y) %floor) (not (aref map x y)))
               (setq maxx (max x maxx)
-                    maxy (max y maxy))
-              (push (list (* x 40) (* y 40) 40 40) *rects*))))
+                    maxy (max y maxy)
+                    *rects* (list* (* x 40) (* y 40) 40 40 *rects*)))))
         (setf (xlib:drawable-width *window*)  (* 40 (1+ maxx))
               (xlib:drawable-height *window*) (* 40 (1+ maxy)))
-        (xlib:shape-combine *window* *rects*)) )))
+        (xlib:shape-combine *window* *rects*))))
+  (xlib:display-force-output *display*))
 
 (defun init-field (&optional (level *level*))
   "Does all initialisation work needed when going to a different level."
@@ -331,7 +332,7 @@
                (update)
                (xlib:display-finish-output *display*)))))))
 
-(defun usage ()
+(defun sokoban-usage ()
   "Print a short description on how to play sokoban."
   (format T "
 
@@ -365,7 +366,7 @@ If you quit sokoban using 'q' the current state will be saved in
 (defun sokoban ()
   (when (or (null *display*) (closed-display-p *display*))
     (init-sokoban)
-    (usage))
+    (sokoban-usage))
   (block event-loop
     (xlib:event-case (*display*)
       (:button-press (code window x y)
@@ -390,7 +391,7 @@ If you quit sokoban using 'q' the current state will be saved in
           (#o153 #|k|# (return-from sokoban 'killed))
           (#o141 #|a|# (stats))
           (#o144 #|d|# (setq *sokoban-debug* (not *sokoban-debug*)))
-          (#o150 #|h|# (usage))
+          (#o150 #|h|# (sokoban-usage))
           (#o156 #|n|#
            (cond ((ready-p)
                   (incf *level*)
