@@ -177,16 +177,17 @@
             ; If the assignment has to be evaluated once for each iteration:
             ; a cons, pointing at the right place in the stepafter-code.
   (requires-stepbefore nil) ; True if the variables can get their values only
-                            ; in the stepbefore-code or initially-code,
+                            ; in the stepbefore-code or preamble,
                             ; false if the first assignment can be merged
                             ; with the initial binding.
   (depends-preceding nil) ; True if everytime=NIL and the values may depend
                           ; on preceding variables, so that these preceding
                           ; variables must get their values no later than in
-                          ; the initially-code.
+                          ; the preamble (= prologue + startup)
+  preamble                      ; cons = location in preamble
   (later-depend nil)) ; True if some later variables depend on these values,
                       ; so that these values
-                      ; must be computed no later than in the initially-code.
+                      ; must be computed no later than in the preamble.
 
 (proclaim '(inline li-vars))
 (defun li-vars (li)
@@ -214,6 +215,13 @@
 ;; Flag whether this variable is used.
 (defvar *used-it*)
 
+;;; (revadd a b c d)) ==
+;;; (SETF A (REVAPPEND B (REVAPPEND C (REVAPPEND D A))))
+(defmacro revadd (place &rest forms)
+  (labels ((expand (rest)
+             (if rest `(revappend ,(car rest) ,(expand (cdr rest))) place)))
+    `(setf ,place ,(expand forms))))
+
 ;; The bulk of the expander.
 (defun expand-loop (*whole* body)
   (let ((body-rest body) ; alle Parse-Funktionen verkürzen body-rest
@@ -233,8 +241,7 @@
         (initializations nil) ; Bindungen: (init ...) (umgedrehte Liste)
         (seen-for-as-= nil) ; schon eine FOR-AS-= Klausel gesehen?
         (seen-endtest nil) ; schon eine FOR-AS Klausel mit Abbruchbedingung gesehen?
-        (initially-code nil) ; initially-Code (umgedrehte Liste)
-        (startup-code nil) ; forms from for ... = ... then and end conditions
+        (preamble nil) ; annotated: ([:INITIALLY|:START] . code) (reversed)
         (stepbefore-code nil) ; Code zum Abbruch vor dem Schleifendurchlauf (umgedrehte Liste)
         (main-code nil) ; Code im Hauptteil der Schleife (umgedrehte Liste)
         (stepafter-code nil) ; Code zur Vorbereitung des nächsten Schleifendurchlaufs (umgedrehte Liste)
@@ -252,6 +259,7 @@
       ((next-kw () ; Schaut, ob als nächstes ein Keyword kommt.
                    ; Wenn ja, wird es geliefert. Wenn nein, Ergebnis NIL.
          (and (consp body-rest) (loop-keywordp (first body-rest))))
+       (preamble (kind &optional form) (car (push (cons kind form) preamble)))
        (cons-forward (form accuvar accufuncsym)
          (let ((tailvar
                 (cdr (or (assoc accuvar accuvar-tailvar-alist)
@@ -516,7 +524,7 @@
        ; - Beim Initialisieren von FOR-AS-= Variablen sind mindestens alle
        ;   vorherigen FOR-AS Variablen initialisiert und deren Abbruch-
        ;   bedingungen abgeprüft.
-       ; Man könnte erst alle Variablen binden und dann im initially-code
+       ; Man könnte erst alle Variablen binden und dann im preamble
        ; die Initialisierungen durchführen. Wir führen demgegenüber zwei
        ; Optimierungen durch:
        ; - Falls vor der FOR-AS Variablen keine FOR-AS-= Klausel kommt,
@@ -550,6 +558,7 @@
            :declspecs nil
            :endtest-forms (list endtest-form)
            :everytime (setq stepafter-code (cons 'NIL stepafter-code))
+           :preamble (preamble :start)
            :requires-stepbefore seen-endtest)))
       ;; Los geht's!
       ; parst: [named name]
@@ -594,7 +603,7 @@
                          main-code)))
                 ((INITIALLY)
                  (pop body-rest)
-                 (push `(PROGN ,@(parse-nonempty-progn kw)) initially-code))
+                 (preamble :INITIALLY `(PROGN ,@(parse-nonempty-progn kw))))
                 ((FINALLY)
                  (pop body-rest)
                  (push (let ((form (parse-unconditional)))
@@ -635,6 +644,7 @@
                           :everytime nil
                           ;; WITH vars should always be bound on top
                           :requires-stepbefore nil ; seen-endtest
+                          :preamble (preamble :start)
                           :depends-preceding t))))
                    ((FOR AS)
                     ; for-as ::= {for | as} for-as-clause {and [{for | as}] for-as-clause}*
@@ -657,6 +667,8 @@
                           (initializations nil)
                           (stepafter nil)
                           (old-seen-endtest seen-endtest)
+                          ;; remember the _CURRENT_ location in preamble
+                          (preamble-entry (preamble :start))
                           (depends-preceding nil))
                       (flet ((note-initialization (initialization)
                                ;; supersedes the outer definition!
@@ -698,6 +710,7 @@
                                          :bindings (destructure pattern (if (eq preposition 'IN) `(CAR ,var) var))
                                          :declspecs new-declspecs
                                          :everytime t
+                                         :preamble (preamble :start)
                                          :requires-stepbefore seen-endtest)))
                                      (push
                                        (list var
@@ -723,7 +736,7 @@
                                      ;; `depends-preceding', because the
                                      ;; `stepafter-code' depends on the order of
                                      ;; the steppings, which forbids moving
-                                     ;; some code from `initially-code' +
+                                     ;; some code from `preamble' +
                                      ;; `stepafter-code' to `stepbefore-code.'
                                      (setq depends-preceding t))
                                    (setq stepafter (revappend (destructure pattern then-form) stepafter))))
@@ -742,6 +755,7 @@
                                        :bindings (destructure pattern `(AREF ,vector-var ,index-var))
                                        :declspecs new-declspecs
                                        :everytime t
+                                       :preamble (preamble :start)
                                        :requires-stepbefore seen-endtest))
                                    (push (list index-var `(1+ ,index-var)) stepafter)))
                                 (BEING
@@ -805,6 +819,7 @@
                                                 :declspecs (unless other-pattern `((IGNORE ,nextother-var)))
                                                 :endtest-forms `((UNLESS ,nextp-var (LOOP-FINISH)))
                                                 :everytime t
+                                                :preamble (preamble :start)
                                                 :requires-stepbefore seen-endtest))
                                               (note-initialization
                                                (make-loop-init
@@ -812,6 +827,7 @@
                                                 :bindings (destructure pattern nextmain-var)
                                                 :declspecs new-declspecs
                                                 :everytime t
+                                                :preamble (preamble :start)
                                                 :requires-stepbefore seen-endtest))
                                               (when other-pattern
                                                 (note-initialization
@@ -820,6 +836,7 @@
                                                   :bindings (destructure other-pattern nextother-var)
                                                   :declspecs nil
                                                   :everytime t
+                                                  :preamble (preamble :start)
                                                   :requires-stepbefore seen-endtest)))))))
                                        ((SYMBOL SYMBOLS PRESENT-SYMBOL PRESENT-SYMBOLS
                                          INTERNAL-SYMBOL INTERNAL-SYMBOLS EXTERNAL-SYMBOL EXTERNAL-SYMBOLS)
@@ -846,6 +863,7 @@
                                             :declspecs nil
                                             :endtest-forms `((UNLESS ,nextp-var (LOOP-FINISH)))
                                             :everytime t
+                                            :preamble (preamble :start)
                                             :requires-stepbefore seen-endtest))
                                           (note-initialization
                                            (make-loop-init
@@ -853,6 +871,7 @@
                                             :bindings (destructure pattern nextsym-var)
                                             :declspecs new-declspecs
                                             :everytime t
+                                            :preamble (preamble :start)
                                             :requires-stepbefore seen-endtest))))))))
                                 (t
                                  (unless (symbolp pattern) (loop-syntax-error kw))
@@ -962,6 +981,7 @@
                           :declspecs (nreverse declspecs)
                           :everytime nil
                           :requires-stepbefore old-seen-endtest
+                          :preamble preamble-entry
                           :depends-preceding depends-preceding))
                       (dolist (initialization (nreverse initializations))
                         (when (li-everytime initialization)
@@ -977,6 +997,7 @@
                           :declspecs nil
                           :everytime nil
                           :requires-stepbefore seen-endtest
+                          :preamble (preamble :start)
                           :depends-preceding t))
                       (push `(SETQ ,var (1- ,var)) stepafter-code)
                       (note-initialization
@@ -999,6 +1020,7 @@
                ;; `*helpvars*' must be bound first thing
                (list (make-loop-init
                       :specform 'LET
+                      :preamble (preamble :start)
                       :bindings
                       (map 'list #'(lambda (var) `(,var NIL)) *helpvars*))))))
         ; `depends-preceding' backpropagation:
@@ -1029,34 +1051,39 @@
                   (make-loop-init
                     :specform 'LET
                     :bindings (default-bindings vars declarations)
+                    :preamble (preamble :start)
                     :declspecs declarations)
                   initializations1)
                 (if everytime
                   (if (li-later-depend initialization)
-                    ;; double code: initially-code and stepafter-code
-                    (setf startup-code (revappend endtest-forms (revappend initforms startup-code))
-                          (cdr everytime) (revappend endtest-forms (revappend initforms (cdr everytime))))
-                    (setq stepbefore-code (revappend endtest-forms (revappend initforms stepbefore-code))))
-                  (setq startup-code (revappend endtest-forms (revappend initforms startup-code)))))
+                    (progn ; double code: preamble and stepafter-code
+                      (revadd (cdr (li-preamble initialization))
+                              endtest-forms initforms)
+                      (revadd (cdr everytime) endtest-forms initforms))
+                    (revadd stepbefore-code endtest-forms initforms))
+                  (revadd (cdr (li-preamble initialization))
+                          endtest-forms initforms)))
               ; Initialisierungsklausel nach initializations1 schaffen:
               (progn
                 (push
                   (make-loop-init
                     :specform name
                     :bindings bindings
+                    :preamble (preamble :start)
                     :declspecs declarations)
                   initializations1)
                 (if everytime
                   (progn
                     ; put the initforms into the stepafter-code only.
-                    (setf (cdr everytime) (revappend initforms (cdr everytime)))
+                    (revadd (cdr everytime) initforms)
                     ; handle the endtest-forms.
                     (if (li-later-depend initialization)
-                      ;; double endtest: initially-code and stepafter-code
-                      (setf startup-code (revappend endtest-forms startup-code)
-                            (cdr everytime) (revappend endtest-forms (cdr everytime)))
-                      (setq stepbefore-code (revappend endtest-forms stepbefore-code))))
-                  (setq startup-code (revappend endtest-forms startup-code)))))))
+                      (progn ; double endtest: preamble and stepafter-code
+                        (revadd (cdr (li-preamble initialization))
+                                endtest-forms)
+                        (revadd (cdr everytime) endtest-forms))
+                      (revadd stepbefore-code endtest-forms)))
+                  (revadd (cdr (li-preamble initialization)) endtest-forms))))))
         (flet ((check-accu-var (var)
                  (when (memq var var-list)
                    (error-of-type 'source-program-error
@@ -1071,43 +1098,51 @@
                         (delete-duplicates accu-vars-nil))
               ,@(mapcar #'(lambda (var) (check-accu-var (car var)) var)
                         (delete-duplicates accu-vars-0 :key #'car)))
+            :preamble (preamble :start)
             :declspecs (nreverse accu-declarations))
            initializations1))
+        (setq preamble
+              (mapcan (lambda (rec)
+                        (case (car rec)
+                          (:start (nreverse
+                                   (mapcar (lambda (f) (cons :start f))
+                                           (cdr rec))))
+                          (:initially (list rec))))
+                      preamble))
         ;; Remove the NIL placeholders in stepafter-code.
         (setq stepafter-code (delete 'NIL stepafter-code))
-        ;; If initially-code and stepafter-code both end in the same
+        ;; If preamble and stepafter-code both end in the same
         ;; forms, drag these forms across the label to stepbefore-code.
-        (unless initially-code
-          (flet ((form-eq (form1 form2) ; Calling EQUAL on user-given forms would be wrong
-                   (or (eql form1 form2)
-                       (and (consp form1) (consp form2)
-                            (eql (length form1) (length form2))
-                            (or (eq (car form1) (car form2))
-                                (and (case (length form1) ((1 3) t))
-                                     (case (car form1) ((SETQ PSETQ) t))
-                                     (case (car form2) ((SETQ PSETQ) t))))
-                            (every #'eq (cdr form1) (cdr form2))))))
-            (loop
-              (unless (and (consp startup-code) (consp stepafter-code)
-                           (form-eq (car startup-code) (car stepafter-code)))
-                (return))
-              (setq stepbefore-code
-                    (nconc stepbefore-code (list (pop stepafter-code))))
-              (pop startup-code))))
+        (flet ((form-eq (form1 form2) ; Calling EQUAL on user-given forms would be wrong
+                 (or (eql form1 form2)
+                     (and (consp form1) (consp form2)
+                          (eql (length form1) (length form2))
+                          (or (eq (car form1) (car form2))
+                              (and (case (length form1) ((1 3) t))
+                                   (case (car form1) ((SETQ PSETQ) t))
+                                   (case (car form2) ((SETQ PSETQ) t))))
+                          (every #'eq (cdr form1) (cdr form2))))))
+          (loop
+            (unless (and (consp preamble) (consp stepafter-code)
+                         (eq :start (caar preamble))
+                         (form-eq (cdar preamble) (car stepafter-code)))
+              (return))
+            (setq stepbefore-code
+                  (nconc stepbefore-code (list (pop stepafter-code))))
+            (pop preamble)))
         ;; Final macroexpansion.
         `(MACROLET ((LOOP-FINISH () (LOOP-FINISH-ERROR)))
            (BLOCK ,block-name
              ,(wrap-initializations initializations1
                 `(MACROLET ((LOOP-FINISH () '(GO END-LOOP)))
                    (TAGBODY
-                     ,@(if startup-code (nreverse startup-code))
-                     ,@(if initially-code (nreverse initially-code))
-                     BEGIN-LOOP
+                     ,@(if preamble (nreverse (mapcar #'cdr preamble)))
+                    BEGIN-LOOP
                      ,@(if stepbefore-code (nreverse stepbefore-code))
                      ,(cons 'PROGN (nreverse main-code))
                      ,@(if stepafter-code (nreverse stepafter-code))
                      (GO BEGIN-LOOP)
-                     END-LOOP
+                    END-LOOP
                      ,@(mapcar #'(lambda (var)
                                    `(SETQ ,var (SYS::LIST-NREVERSE ,var)))
                                accu-vars-nreverse)
